/*
  This allows you to rerender with a selector or an element.
  The component attached to the node will be rerendered.
*/
import { rerender, getForgoState } from "forgo";
/*
  Namespaces
*/
const HTML_NAMESPACE = "http://www.w3.org/1999/xhtml";
const MATH_NAMESPACE = "http://www.w3.org/1998/Math/MathML";
const SVG_NAMESPACE = "http://www.w3.org/2000/svg";
/*
  The element types we care about.
  As defined by the standards.
*/
const ELEMENT_NODE_TYPE = 1;
const ATTRIBUTE_NODE_TYPE = 2;
const TEXT_NODE_TYPE = 3;
const documentObject = globalThis ? globalThis.document : document;
const windowObject = globalThis ? globalThis : window;
let env = {
    window: windowObject,
    document: documentObject,
};
function isString(val) {
    return typeof val === "string";
}
export function setCustomEnv(value) {
    env = value;
}
export function rerenderElement(elementSelector, props) {
    let element = (isString(elementSelector)
        ? env.document.querySelector(elementSelector)
        : elementSelector);
    if (element) {
        if (element.nodeType === ELEMENT_NODE_TYPE) {
            const state = getForgoState(element);
            if (state && state.components.length) {
                return rerender(state.components[0].args.element, props);
            }
            else {
                throw new Error("Could not find an attached component to rerender.");
            }
        }
        else {
            throw new Error("The node passed to rerenderElement() should be an HTML element.");
        }
    }
    else {
        throw new Error(`The rerenderElement() function was called on a non-element (${typeof elementSelector === "string"
            ? elementSelector
            : elementSelector === null || elementSelector === void 0 ? void 0 : elementSelector.tagName}).`);
    }
}
//# sourceMappingURL=index.js.map